@@ -62,6 +62,9 @@ static const QByteArray USER_AGENT("Wget/1.13.4 (linux-gnu)");
 static const QString ALL_ARTICLES_SUBSCRIPTION_ID("all_articles");
 static const QString FAVOURITES_SUBSCRIPTION_ID("favourite_articles");
 
+// Articles
+static const int MAX_ARTICLES = 200;
+
 // Version
 static const QString VERSION_NUMBER("1.0.0");
 
